Package: rtForecastEval
Title: Evaluate the Discrepancy Between Two Real-Time Updated
        Probabilistic Forecasts
Version: 0.1.0
Authors@R: c(
    person(given = "Chi-Kuang", family = "Yeh",
             role = c("aut", "cre"),
             email = "cyeh@gsu.edu",
             comment = c(ORCID = "0000-0001-7057-2096")),
    person(given = "Gregory", family = "Rice",
           role =  c("ctb", "ths")),
    person(given = "Joel A.", family = "Dubin",
           role =  c("ctb", "ths"))     
     )
Description: 
    Methods from Yeh, Rice, and Dubin (2022) <doi:10.1080/00031305.2021.1967781> for comparing two continuously updated probabilistic
    forecasts under squared (Brier) loss: pointwise loss and variance, a global delta test (Monte Carlo p-values), simulation designs, and a naive
    pointwise band plot.
URL: https://github.com/chikuang/rtForecastEval
BugReports: https://github.com/chikuang/rtForecastEval/issues
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: dplyr (>= 1.0.1), ggplot2 (>= 3.3.2), stats (>= 4.0.3), tidyr
        (>= 1.1.1), RSpectra (>= 0.16.0), rlist (>= 0.4.6.1), purrr (>=
        0.3.4), MASS (>= 7.3-52)
Suggests: pkgload (>= 1.2.0), knitr (>= 1.28.0), readr (>= 1.3.1),
        reshape2 (>= 1.4.4), rmarkdown (>= 2.1.0), gridExtra (>=
        2.3.0), tibble (>= 3.0.3), sde
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-24 02:21:34 UTC; chikuang
Author: Chi-Kuang Yeh [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7057-2096>),
  Gregory Rice [ctb, ths],
  Joel A. Dubin [ctb, ths]
Maintainer: Chi-Kuang Yeh <cyeh@gsu.edu>
Repository: CRAN
Date/Publication: 2026-04-28 18:30:02 UTC
Built: R 4.5.2; ; 2026-05-01 19:01:54 UTC; unix

Package: rtForecastEval
Title: Evaluate the Discrepancy Between Two Real-Time Updated
        Probabilistic Forecasts
Version: 0.1.0
Authors@R: c(
    person(given = "Chi-Kuang", family = "Yeh",
             role = c("aut", "cre"),
             email = "cyeh@gsu.edu",
             comment = c(ORCID = "0000-0001-7057-2096")),
    person(given = "Gregory", family = "Rice",
           role =  c("ctb", "ths")),
    person(given = "Joel A.", family = "Dubin",
           role =  c("ctb", "ths"))     
     )
Description: 
    Methods from Yeh, Rice, and Dubin (2022) <doi:10.1080/00031305.2021.1967781> for comparing two continuously updated probabilistic
    forecasts under squared (Brier) loss: pointwise loss and variance, a global delta test (Monte Carlo p-values), simulation designs, and a naive
    pointwise band plot.
URL: https://github.com/chikuang/rtForecastEval
BugReports: https://github.com/chikuang/rtForecastEval/issues
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: dplyr (>= 1.0.1), ggplot2 (>= 3.3.2), stats (>= 4.0.3), tidyr
        (>= 1.1.1), RSpectra (>= 0.16.0), rlist (>= 0.4.6.1), purrr (>=
        0.3.4), MASS (>= 7.3-52)
Suggests: pkgload (>= 1.2.0), knitr (>= 1.28.0), readr (>= 1.3.1),
        reshape2 (>= 1.4.4), rmarkdown (>= 2.1.0), gridExtra (>=
        2.3.0), tibble (>= 3.0.3), sde
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-24 02:21:34 UTC; chikuang
Author: Chi-Kuang Yeh [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7057-2096>),
  Gregory Rice [ctb, ths],
  Joel A. Dubin [ctb, ths]
Maintainer: Chi-Kuang Yeh <cyeh@gsu.edu>
Repository: CRAN
Date/Publication: 2026-04-28 18:30:02 UTC
Built: R 4.6.0; ; 2026-05-01 22:15:08 UTC; unix

# Generated by roxygen2: do not edit by hand

export(calc_L_s2)
export(calc_Z)
export(calc_eig)
export(calc_pval)
export(df_gen)
export(lin_interp)
export(plot_pcb)
import(dplyr)
import(ggplot2)
import(purrr)
import(tidyr)
importFrom(MASS,mvrnorm)
importFrom(rlist,list.rbind)
importFrom(stats,approx)
importFrom(stats,ecdf)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,rchisq)
importFrom(stats,runif)

# rtForecastEval: EVALuating Real-Time Probabilistic Forecast

April 23, 2026

<!-- After editing README.qmd, run: Rscript tools/render-readme.R README.qmd (fixes Mermaid code fences for GitHub). -->

<!-- badges: start -->

[![R-CMD-check](https://github.com/chikuang/rtForecastEval/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/chikuang/rtForecastEval/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

**Chi-Kuang Yeh** (Georgia State University)
[![ORCID](https://img.shields.io/badge/ORCID-0000--0001--7057--2096-A6CE39?logo=orcid.png)](https://orcid.org/0000-0001-7057-2096)

**Gregory Rice** (University of Waterloo)

**Joel A. Dubin** (University of Waterloo)

## Overview

**rtForecastEval** implements the methodology in [Yeh, Rice, and Dubin
(2022)](https://doi.org/10.1080/00031305.2021.1967781) ([preprint
arXiv:2010.00781](https://arxiv.org/abs/2010.00781),
[PDF](https://arxiv.org/pdf/2010.00781.pdf)). The paper develops tools
for **continuously updated** probabilistic forecasts: calibration-style
summaries, **skill** (relative performance of two forecasters) via a
global **delta test** on **L²\[0,1\]** under squared (Brier) loss, and
simple **pointwise** inference for the mean loss difference over time.

This package ships the statistical core used in the paper’s
**simulation** sections. Full **NBA** replication (scraped data,
calibration surfaces, competitor models) lives in a separate analysis
repository and is not bundled here.

## Package contents

| Topic | Functions |
|----|----|
| Simulation designs | `df_gen()` — requires the [**sde**](https://CRAN.R-project.org/package=sde) package at runtime |
| Pointwise loss / variance | `calc_L_s2()`, `lin_interp()` |
| Global delta test | `calc_Z()`, `calc_eig()`, `calc_pval()` |
| Plotting | `plot_pcb()` (naive pointwise **skill** band); README below also shows **reliability** plots and a **calibration-difference** curve between forecasters |

After installation, run `help(package = "rtForecastEval")` or `?df_gen`
for full documentation.

## Mind map and workflow

The diagrams below use [Mermaid](https://mermaid.js.org/). They render
on GitHub; in other viewers you may see the source only.

### Mind map (package scope)

``` mermaid
mindmap
  root((rtForecastEval))
    Paper
      Yeh Rice Dubin 2022
      Real-time probabilistic forecasts
    Inputs
      df_gen simulation
      Your own long-format data
    Global skill test
      calc_Z delta statistic
      calc_eig covariance spectrum
      calc_pval MC p-values
    Pointwise skill
      calc_L_s2 loss and variance
      plot_pcb naive band
    Other
      lin_interp time grid
```

### Analysis workflow

``` mermaid
flowchart TD
  subgraph src["Data sources"]
    A["df_gen()<br>(simulation)"]
    B["Your forecasts<br>(long format)"]
  end
  C["Long tibble:<br>game, grid, Y,<br>phat_A, phat_B"]
  D["Centered / non-centered<br>differences"]
  E["calc_Z()"]
  F["calc_eig()"]
  G["calc_pval()"]
  H["calc_L_s2()"]
  P["plot_pcb()"]
  A --> C
  B --> C
  C --> D
  D --> E
  D --> F
  E --> G
  F --> G
  D --> H
  H --> P
```

## Installation

``` r
install.packages("pak")
pak::pak("chikuang/rtForecastEval")
```

## Minimal example

After installing the package (and **sde** for `df_gen()`), load
**dplyr**, **tidyr**, **ggplot2**, and **rtForecastEval**. The chunks
below **execute** when you render `README.qmd`
(e.g. `quarto render README.qmd` or
`Rscript tools/render-readme.R README.qmd`) with the package available —
for example `devtools::load_all()` from a local clone or
`library(rtForecastEval)` after `pak::pak("chikuang/rtForecastEval")`.

``` r
library(dplyr)
```


    Attaching package: 'dplyr'

    The following objects are masked from 'package:stats':

        filter, lag

    The following objects are masked from 'package:base':

        intersect, setdiff, setequal, union

``` r
library(tidyr)
library(ggplot2)
library(rtForecastEval)
library(sde)
```

    Loading required package: MASS


    Attaching package: 'MASS'

    The following object is masked from 'package:dplyr':

        select

    Loading required package: stats4

    Loading required package: fda

    Loading required package: splines

    Loading required package: fds

    Loading required package: rainbow

    Loading required package: pcaPP

    Loading required package: RCurl


    Attaching package: 'RCurl'

    The following object is masked from 'package:tidyr':

        complete

    Loading required package: deSolve


    Attaching package: 'fda'

    The following object is masked from 'package:graphics':

        matplot

    The following object is masked from 'package:datasets':

        gait

    Loading required package: zoo


    Attaching package: 'zoo'

    The following objects are masked from 'package:base':

        as.Date, as.Date.numeric

    sde 2.0.21

    Companion package to the book

    'Simulation and Inference for Stochastic Differential Equations With R Examples'

    Iacus, Springer NY, (2008)

    To check the errata corrige of the book, type vignette("sde.errata")

``` r
set.seed(777)
nsamp <- 40   # interior steps; df_gen() uses a grid of nsamp + 1 points on [0, 1]
ngame <- 1000  # independent replicates (“games”); larger n → more games per calibration bin
D <- 8        # leading eigenvalues for the chi-square approximation
N_MC <- 2000  # Monte Carlo draws for the p-value

df_equ <- df_gen(N = nsamp, Ngame = ngame) |>
  group_by(grid) |>
  mutate(
    p_bar_12 = mean(phat_A - phat_B),
    diff_non_cent = phat_A - phat_B,
    diff_cent = phat_A - phat_B - p_bar_12
  ) |>
  ungroup()

ZZ <- calc_Z(df_equ, "phat_A", "phat_B", "Y", nsamp = nsamp, ngame = ngame)
eig <- calc_eig(df_equ, n_eig = D, ngame = ngame, nsamp = nsamp, cent = FALSE)
out <- calc_pval(ZZ, eig, quan = c(0.9, 0.95, 0.99), n_MC = N_MC)

Ltab <- calc_L_s2(df_equ, "phat_A", "phat_B")
plot_pcb(Ltab)
```

![](README_files/figure-commonmark/unnamed-chunk-3-1.png)

``` r
c(Z = ZZ, p_value = out$p_val, out$quantile)
```

             Z    p_value        90%        95%        99% 
    0.05470115 0.72300000 0.40628731 0.58446258 0.91682971 

### Pointwise skill plot and a simple calibration (reliability) view

`plot_pcb()` plots the **pointwise mean loss difference** (relative
**skill**) with a naive normal band — not a classical **calibration**
diagram.

A complementary check is **marginal calibration** at one time: bin
predicted probabilities at a single grid (here, closest to mid-game, **t
≈ 0.5**) and compare mean outcome to mean forecast (reliability
diagram). **Grey vertical segments** show a **95% central range** for
the **bin mean of Y** under **H₀** (perfect calibration in the bin):
under the null, outcomes are Bernoulli with probability equal to the
bin’s mean forecast **p̄**, so the bin sum is **Binomial(n, p̄)** and the
segment endpoints use **exact** quantiles of that binomial (no normal
approximation or asymmetric clipping to **\[0, 1\]**, which can make
bands look “anchored” at **y = 0**). Full **calibration surfaces** from
the paper are in the NBA replication code; this is a minimal ggplot
using the same long-format columns (`phat_A`, `phat_B`, `Y`, `grid`).

``` r
# Uses `df_equ` from the chunk above
g_mid <- df_equ |>
  distinct(grid) |>
  slice_min(order_by = abs(grid - 0.5), n = 1) |>
  pull(grid)

slice_t <- df_equ |>
  filter(grid == g_mid) |>
  transmute(game, Y, phat_A, phat_B)

rel_long <- slice_t |>
  pivot_longer(c(phat_A, phat_B), names_to = "forecaster", values_to = "phat") |>
  mutate(forecaster = ifelse(forecaster == "phat_A", "Forecaster A", "Forecaster B"))

rel_binned <- rel_long |>
  group_by(forecaster) |>
  mutate(bin = ntile(phat, 10)) |>
  group_by(forecaster, bin) |>
  summarise(
    mean_forecast = mean(phat),
    mean_outcome = mean(Y),
    n_games = n(),
    .groups = "drop"
  ) |>
  mutate(
    p_bin = pmin(pmax(mean_forecast, 0), 1),
    ci_lo = stats::qbinom(0.025, size = n_games, prob = p_bin) / n_games,
    ci_hi = stats::qbinom(0.975, size = n_games, prob = p_bin) / n_games
  )

ggplot(rel_binned, aes(mean_forecast, mean_outcome)) +
  geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "grey45") +
  geom_errorbar(
    aes(x = mean_forecast, ymin = ci_lo, ymax = ci_hi),
    width = 0.02,
    color = "grey55",
    alpha = 0.85,
    linewidth = 0.35
  ) +
  geom_point(aes(size = n_games), alpha = 0.9, color = "darkred") +
  facet_wrap(~forecaster, nrow = 1) +
  coord_fixed(xlim = c(0, 1), ylim = c(0, 1), ratio = 1) +
  labs(
    title = "Binned reliability (calibration) at one grid",
    subtitle = paste0(
      "Grid = ", signif(g_mid, 3),
      "; grey: exact 95% central range for mean(Y) under H0 (Binomial; see text)"
    ),
    x = "Mean forecast in bin",
    y = "Mean outcome (Y) in bin",
    size = "Games"
  ) +
  theme_minimal(base_size = 12) +
  theme(
    panel.grid.minor = ggplot2::element_blank(),
    plot.title = ggplot2::element_text(face = "bold")
  )
```

![](README_files/figure-commonmark/unnamed-chunk-4-1.png)

### Reliability with shared bins (two standard diagrams)

The facet plot above uses **separate** decile bins per forecaster, so
bins are **not** aligned across A vs B. Here we bin the **same games**
with **equal-width** cuts of the **midpoint**
**$m = (phat_A + phat_B) / 2$** (same definition as in the code chunk).
For each bin we compute **mean(Y)** and each forecaster’s mean forecast
in that bin, then draw **one classical reliability diagram per
forecaster** (horizontal axis: mean forecast; vertical: mean outcome;
both in **\[0, 1\]**). **Grey vertical** segments are the **exact
Binomial** **95%** central range for **mean(Y)** under **$H_0$** at that
forecaster’s bin mean forecast (same construction as the first figure).
This avoids overlaying two forecasters in one square, which is easy to
misread.

``` r
# Same grid as above; shared midpoint bins for the next two figures
cal_bins <- slice_t |>
  mutate(mid = (phat_A + phat_B) / 2) |>
  mutate(bin = cut(mid, breaks = seq(0, 1, length.out = 11), include.lowest = TRUE))

rel_joint <- cal_bins |>
  group_by(bin) |>
  summarise(
    mean_forecast_A = mean(phat_A),
    mean_forecast_B = mean(phat_B),
    mean_outcome = mean(Y),
    n_games = n(),
    .groups = "drop"
  ) |>
  filter(!is.na(bin)) |>
  mutate(
    p_A = pmin(pmax(mean_forecast_A, 0), 1),
    p_B = pmin(pmax(mean_forecast_B, 0), 1),
    ci_lo_A = stats::qbinom(0.025, size = n_games, prob = p_A) / n_games,
    ci_hi_A = stats::qbinom(0.975, size = n_games, prob = p_A) / n_games,
    ci_lo_B = stats::qbinom(0.025, size = n_games, prob = p_B) / n_games,
    ci_hi_B = stats::qbinom(0.975, size = n_games, prob = p_B) / n_games
  )

rel_joint_long <- rel_joint |>
  tidyr::pivot_longer(
    c(mean_forecast_A, mean_forecast_B),
    names_to = "which",
    names_prefix = "mean_forecast_",
    values_to = "mean_forecast"
  ) |>
  mutate(
    forecaster = ifelse(which == "A", "Forecaster A", "Forecaster B"),
    ci_lo = ifelse(which == "A", ci_lo_A, ci_lo_B),
    ci_hi = ifelse(which == "A", ci_hi_A, ci_hi_B)
  )

ggplot(rel_joint_long, aes(mean_forecast, mean_outcome)) +
  geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "grey45") +
  geom_errorbar(
    aes(x = mean_forecast, ymin = ci_lo, ymax = ci_hi),
    width = 0.02,
    color = "grey55",
    alpha = 0.85,
    linewidth = 0.4
  ) +
  geom_point(aes(color = forecaster, size = n_games), alpha = 0.9) +
  facet_wrap(~forecaster, nrow = 1) +
  coord_fixed(xlim = c(0, 1), ylim = c(0, 1), ratio = 1) +
  scale_color_manual(values = c("Forecaster A" = "steelblue", "Forecaster B" = "orangered")) +
  labs(
    title = "Reliability: shared midpoint bins (one classical diagram per forecaster)",
    subtitle = "Bins from cut((phat_A + phat_B) / 2); grey = exact 95% Binomial H0 range for mean(Y)",
    x = "Mean forecast in bin",
    y = "Mean outcome (Y) in bin",
    color = NULL,
    size = "Games"
  ) +
  theme_minimal(base_size = 12) +
  theme(
    panel.grid.minor = ggplot2::element_blank(),
    plot.title = ggplot2::element_text(face = "bold"),
    legend.position = "right"
  ) +
  guides(color = "none")
```

![](README_files/figure-commonmark/unnamed-chunk-5-1.png)

### Comparing calibration between forecasters (error curves)

The reliability diagram shows **levels** on the 45° plot; the plot below
shows the same bins but **calibration errors** **mean(Y) − mean
forecast** vs bin midpoint. Curves near **0** are better calibrated; the
**dashed** line is **mean(phat_B) − mean(phat_A)** within each bin.

``` r
cal_compare <- cal_bins |>
  group_by(bin) |>
  summarise(
    mid_hat = mean(mid),
    cal_err_A = mean(Y) - mean(phat_A),
    cal_err_B = mean(Y) - mean(phat_B),
    n_games = n(),
    .groups = "drop"
  ) |>
  filter(!is.na(bin))

cal_long <- cal_compare |>
  pivot_longer(
    c(cal_err_A, cal_err_B),
    names_to = "which",
    values_to = "cal_err"
  ) |>
  mutate(
    which = ifelse(which == "cal_err_A", "A: mean(Y) - phat_A", "B: mean(Y) - phat_B")
  )

ggplot(cal_long, aes(mid_hat, cal_err, color = which)) +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey70") +
  geom_line(linewidth = 0.85) +
  geom_point(aes(size = n_games), alpha = 0.85) +
  geom_line(
    data = cal_compare,
    aes(mid_hat, cal_err_B - cal_err_A),
    inherit.aes = FALSE,
    color = "grey35",
    linetype = "dashed",
    linewidth = 0.65
  ) +
  labs(
    title = "Calibration comparison (same midpoint bins)",
    subtitle = "Solid: mean calibration error per forecaster; dashed: mean(phat_B) - mean(phat_A)",
    x = "Mean (phat_A + phat_B) / 2 in bin",
    y = "mean(Y) - mean(phat)",
    color = "Curve",
    size = "Games"
  ) +
  theme_minimal(base_size = 12) +
  theme(
    panel.grid.minor = ggplot2::element_blank(),
    plot.title = ggplot2::element_text(face = "bold"),
    legend.position = "bottom"
  )
```

![](README_files/figure-commonmark/unnamed-chunk-6-1.png)

For the full workflow (explicit linear algebra vs. wrappers, larger
grids, centered eigenvalues, and the same figures with narrative), open
the vignette:

``` r
vignette("rtForecastEval-vignette", package = "rtForecastEval")
```

## Roadmap

- [ ] Optional pkgdown site and CRAN release
- [ ] Optional **Rcpp** acceleration for heavy eigenvalue / Monte Carlo
  loops

## References

- Yeh, C.-K., Rice, G. & Dubin, J. A. (2022). [Evaluating real-time
  probabilistic forecasts with application to National Basketball
  Association outcome
  prediction](https://www.tandfonline.com/doi/full/10.1080/00031305.2021.1967781).
  *The American Statistician*, *76*, 214–223. DOI
  [10.1080/00031305.2021.1967781](https://doi.org/10.1080/00031305.2021.1967781).
- Preprint: [arXiv:2010.00781](https://arxiv.org/abs/2010.00781)
  ([PDF](https://arxiv.org/pdf/2010.00781.pdf)).

## License

GPL-3 — see [DESCRIPTION](DESCRIPTION).

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
.old_opts <- options(digits = 3)

## ----workflow-schematic, echo=FALSE, warning = FALSE, message= FALSE, fig.width=8, fig.height=4, fig.cap="Typical workflow: global test (top) and pointwise loss plot (bottom)."----
library(ggplot2)

box <- function(x, y, w, h, label) {
  data.frame(
    xmin = x - w / 2, xmax = x + w / 2,
    ymin = y - h / 2, ymax = y + h / 2,
    label = label, x = x, y = y
  )
}

top <- rbind(
  box(1.2, 5, 1.1, 0.55, "Data\n(df_gen or yours)"),
  box(3.1, 5, 1.0, 0.55, "Long format +\ncentered diffs"),
  box(4.9, 5.45, 0.85, 0.45, "calc_Z"),
  box(4.9, 4.55, 0.85, 0.45, "calc_eig"),
  box(6.8, 5, 0.95, 0.55, "calc_pval")
)
bot <- rbind(
  box(3.1, 2.6, 1.0, 0.55, "calc_L_s2"),
  box(5.0, 2.6, 0.85, 0.55, "plot_pcb")
)
rects <- rbind(top, bot)

seg <- data.frame(
  x = c(1.75, 2.65, 4.45, 4.45, 5.35, 5.35, 3.1, 3.65),
  y = c(5, 5, 5.45, 4.55, 5.45, 4.55, 4.45, 2.6),
  xend = c(2.6, 3.6, 5.35, 5.35, 6.35, 6.35, 3.1, 4.55),
  yend = c(5, 5, 5.45, 4.55, 5, 5, 3.15, 2.6)
)

ggplot() +
  geom_rect(
    data = rects,
    aes(xmin = xmin, xmax = xmax, ymin = ymin, ymax = ymax),
    fill = "grey96", color = "grey35", size = 0.35
  ) +
  geom_text(data = rects, aes(x = x, y = y, label = label), size = 3) +
  geom_segment(
    data = seg,
    aes(x = x, y = y, xend = xend, yend = yend),
    arrow = grid::arrow(length = grid::unit(0.12, "cm"), type = "closed"),
    size = 0.35,
    color = "grey25"
  ) +
  annotate("text", x = 4.9, y = 5.95, label = "Global delta test (chi-square approx.)", size = 3.2, fontface = "italic") +
  annotate("text", x = 4.0, y = 1.85, label = "Pointwise mean loss + naive band", size = 3.2, fontface = "italic") +
  coord_cartesian(xlim = c(0.3, 7.5), ylim = c(1.5, 6.2), clip = "off") +
  theme_void() +
  theme(plot.margin = grid::unit(c(12, 8, 8, 8), "pt"))

## ----load, message = FALSE, include = FALSE-----------------------------------
require(dplyr)
require(tidyr)
require(gridExtra)
require(RSpectra)
require(rlist)

## ----include = FALSE----------------------------------------------------------
library(rtForecastEval)

## -----------------------------------------------------------------------------
library(ggplot2)
library(tibble)
library(MASS)
nsamp <- 100 # number of in-game events
ngame <- 200 # number of games (README example uses the same for comparable figures)

#' Parameter for generating the eigenvalues, and p-values
D <- 10 # Number of eigenvalues to keep
N_MC <- 5000 # for simulating the p-value
L <- function(x, y) {
  return((x - y) ^ 2)
}

# Data generation ---------------------------------------------------------
df_equ <- df_gen(N = nsamp, Ngame = ngame) %>%
  group_by(grid) %>%
  mutate(
    p_bar_12 = mean(phat_A - phat_B),
    diff_non_cent = phat_A - phat_B,
    diff_cent = phat_A - phat_B - p_bar_12
  ) %>% ungroup()

# Apply our test (explicit construction, as in utility.R) ----------------

Z <- df_equ %>% group_by(grid) %>%
  summarise(delta_n = mean(L(phat_A, Y) - L(phat_B, Y))) %>%
  {
    sum((.)$delta_n^2) / nsamp * ngame
  }

temp <- df_equ %>% group_split(grid, .keep = FALSE)

eigV_hat <- lapply(1:nsamp, function(i) {
  sapply(1:nsamp, function(j) {
    as.numeric(temp[[i]]$diff_non_cent %*% temp[[j]]$diff_non_cent / ngame)
  })
}) %>% list.rbind() %>% {
  RSpectra::eigs_sym(
    A = (.),
    k = D,
    which = "LM",
    opts = list(retvec = FALSE)
  )$values
} %>%
  {
    (.)/nsamp
  }

eigV_til <- lapply(1:nsamp, function(i) {
  sapply(1:nsamp, function(j) {
    as.numeric(temp[[i]]$diff_cent %*% temp[[j]]$diff_cent / ngame)
  })
}) %>% list.rbind() %>% {
  RSpectra::eigs_sym(
    A = (.),
    k = D,
    which = "LM",
    opts = list(retvec = FALSE)
  )$values
} %>%
  {
    (.)/nsamp
  }

MC_hat <- sapply(1:N_MC, function(x) {
  crossprod(eigV_hat, rchisq(D, df = 1))
})

q_90_hat <- quantile(MC_hat, 0.90)
q_95_hat <- quantile(MC_hat, 0.95)
q_99_hat <- quantile(MC_hat, 0.99)

MC_til <- sapply(1:N_MC, function(x) {
  crossprod(eigV_til, rchisq(D, df = 1))
})

q_90_til <- quantile(MC_til, 0.90)
q_95_til <- quantile(MC_til, 0.95)
q_99_til <- quantile(MC_til, 0.99)

p_hat <- 1 - ecdf(MC_hat)(Z)

tibble(
  type = c("non-center", "center"),
  Z = rep(Z, 2),
  "pval" = c(p_hat, p_hat),
  "90%" = c(q_90_hat, q_90_til),
  "95%" = c(q_95_hat, q_95_til),
  "99%" = c(q_99_hat, q_99_til)
)

## ----function wrappers, fig.width = 7, fig.height = 4.2, fig.cap = "Pointwise mean loss difference (A vs B) with naive normal band — simulation setting. This is a skill curve, not a calibration diagram."----
to_center <- FALSE

ZZ <- calc_Z(df = df_equ, pA = "phat_A", pB = "phat_B", Y = "Y", nsamp = nsamp, ngame = ngame)
eigg <- calc_eig(
  df = df_equ, n_eig = D, ngame = ngame,
  nsamp = nsamp, grid = "grid", cent = to_center
)
oh <- calc_pval(ZZ, eig = eigg, quan = c(0.90, 0.95, 0.99), n_MC = N_MC)

temp <- calc_L_s2(df = df_equ, pA = "phat_A", pB = "phat_B")

plot_pcb(df = temp)

tibble(
  type = ifelse(to_center, "center", "non-center"),
  Z = ZZ,
  pval = oh$p_val,
  "90%" = oh$quantile[1],
  "95%" = oh$quantile[2],
  "99%" = oh$quantile[3]
)

## ----calibration-reliability, fig.width = 7.5, fig.height = 4, fig.cap = "Binned reliability diagram at a fixed grid (closest to 0.5): mean outcome vs mean forecast for A and B; grey bars are exact 95% central Binomial range for mean(Y) under H0. Points near the diagonal indicate better marginal calibration at that time."----
g_mid <- df_equ %>%
  distinct(grid) %>%
  slice_min(order_by = abs(grid - 0.5), n = 1) %>%
  pull(grid)

slice_t <- df_equ %>%
  filter(grid == g_mid) %>%
  transmute(
    game,
    Y,
    phat_A,
    phat_B
  )

rel_long <- slice_t %>%
  tidyr::pivot_longer(c(phat_A, phat_B), names_to = "forecaster", values_to = "phat") %>%
  mutate(forecaster = ifelse(forecaster == "phat_A", "Forecaster A", "Forecaster B"))

rel_binned <- rel_long %>%
  group_by(forecaster) %>%
  mutate(bin = ntile(phat, 10)) %>%
  group_by(forecaster, bin) %>%
  summarise(
    mean_forecast = mean(phat),
    mean_outcome = mean(Y),
    n_games = dplyr::n(),
    .groups = "drop"
  ) %>%
  mutate(
    p_bin = pmin(pmax(mean_forecast, 0), 1),
    ci_lo = stats::qbinom(0.025, size = n_games, prob = p_bin) / n_games,
    ci_hi = stats::qbinom(0.975, size = n_games, prob = p_bin) / n_games
  )

ggplot(rel_binned, aes(mean_forecast, mean_outcome)) +
  geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "grey45") +
  geom_errorbar(
    aes(x = mean_forecast, ymin = ci_lo, ymax = ci_hi),
    width = 0.02,
    color = "grey55",
    alpha = 0.85,
    linewidth = 0.35
  ) +
  geom_point(aes(size = n_games), alpha = 0.9, color = "darkred") +
  facet_wrap(~forecaster, nrow = 1) +
  coord_fixed(xlim = c(0, 1), ylim = c(0, 1), ratio = 1) +
  labs(
    title = "Binned reliability (calibration) at one grid",
    subtitle = paste0(
      "Grid = ", signif(g_mid, 3),
      " (closest to 0.5); grey: exact 95% Binomial range for mean(Y) under H0"
    ),
    x = "Mean forecast in bin",
    y = "Mean outcome (Y) in bin",
    size = "Games"
  ) +
  theme_minimal(base_size = 12) +
  theme(
    panel.grid.minor = ggplot2::element_blank(),
    plot.title = ggplot2::element_text(face = "bold")
  )

## ----reliability-joint-difference, fig.width = 8, fig.height = 4.2, fig.cap = "Shared midpoint bins: two standard reliability diagrams (A and B); grey bars = exact 95% Binomial range for mean(Y) under H0."----
cal_bins <- slice_t %>%
  mutate(mid = (phat_A + phat_B) / 2) %>%
  mutate(bin = cut(mid, breaks = seq(0, 1, length.out = 11), include.lowest = TRUE))

rel_joint <- cal_bins %>%
  group_by(bin) %>%
  summarise(
    mean_forecast_A = mean(phat_A),
    mean_forecast_B = mean(phat_B),
    mean_outcome = mean(Y),
    n_games = dplyr::n(),
    .groups = "drop"
  ) %>%
  filter(!is.na(bin)) %>%
  mutate(
    p_A = pmin(pmax(mean_forecast_A, 0), 1),
    p_B = pmin(pmax(mean_forecast_B, 0), 1),
    ci_lo_A = stats::qbinom(0.025, size = n_games, prob = p_A) / n_games,
    ci_hi_A = stats::qbinom(0.975, size = n_games, prob = p_A) / n_games,
    ci_lo_B = stats::qbinom(0.025, size = n_games, prob = p_B) / n_games,
    ci_hi_B = stats::qbinom(0.975, size = n_games, prob = p_B) / n_games
  )

rel_joint_long <- rel_joint %>%
  tidyr::pivot_longer(
    c(mean_forecast_A, mean_forecast_B),
    names_to = "which",
    names_prefix = "mean_forecast_",
    values_to = "mean_forecast"
  ) %>%
  mutate(
    forecaster = ifelse(which == "A", "Forecaster A", "Forecaster B"),
    ci_lo = ifelse(which == "A", ci_lo_A, ci_lo_B),
    ci_hi = ifelse(which == "A", ci_hi_A, ci_hi_B)
  )

ggplot(rel_joint_long, aes(mean_forecast, mean_outcome)) +
  geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "grey45") +
  geom_errorbar(
    aes(x = mean_forecast, ymin = ci_lo, ymax = ci_hi),
    width = 0.02,
    color = "grey55",
    alpha = 0.85,
    linewidth = 0.4
  ) +
  geom_point(aes(color = forecaster, size = n_games), alpha = 0.9) +
  facet_wrap(~forecaster, nrow = 1) +
  coord_fixed(xlim = c(0, 1), ylim = c(0, 1), ratio = 1) +
  scale_color_manual(values = c("Forecaster A" = "steelblue", "Forecaster B" = "orangered")) +
  labs(
    title = "Reliability: shared midpoint bins (one classical diagram per forecaster)",
    subtitle = "Bins from cut((phat_A + phat_B) / 2); grey = exact 95% Binomial H0 range for mean(Y)",
    x = "Mean forecast in bin",
    y = "Mean outcome (Y) in bin",
    color = NULL,
    size = "Games"
  ) +
  theme_minimal(base_size = 12) +
  theme(
    panel.grid.minor = ggplot2::element_blank(),
    plot.title = ggplot2::element_text(face = "bold"),
    legend.position = "right"
  ) +
  guides(color = "none")

## ----calibration-compare-forecasters, fig.width = 7.5, fig.height = 4.5, fig.cap = "Mean calibration error per forecaster (solid) and difference in mean forecasts (dashed), binned by (phat_A + phat_B) / 2 at the same grid as the reliability figure."----
cal_compare <- cal_bins %>%
  group_by(bin) %>%
  summarise(
    mid_hat = mean(mid),
    cal_err_A = mean(Y) - mean(phat_A),
    cal_err_B = mean(Y) - mean(phat_B),
    n_games = dplyr::n(),
    .groups = "drop"
  ) %>%
  filter(!is.na(bin))

cal_long <- cal_compare %>%
  tidyr::pivot_longer(
    c(cal_err_A, cal_err_B),
    names_to = "which",
    values_to = "cal_err"
  ) %>%
  mutate(
    which = ifelse(which == "cal_err_A", "A: mean(Y) - phat_A", "B: mean(Y) - phat_B")
  )

ggplot(cal_long, aes(mid_hat, cal_err, color = which)) +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey70") +
  geom_line(linewidth = 0.85) +
  geom_point(aes(size = n_games), alpha = 0.85) +
  geom_line(
    data = cal_compare,
    aes(mid_hat, cal_err_B - cal_err_A),
    inherit.aes = FALSE,
    color = "grey35",
    linetype = "dashed",
    linewidth = 0.65
  ) +
  labs(
    title = "Calibration comparison (same midpoint bins)",
    subtitle = "Solid: mean calibration error per forecaster; dashed: mean(phat_B) - mean(phat_A)",
    x = "Mean (phat_A + phat_B) / 2 in bin",
    y = "mean(Y) - mean(phat)",
    color = "Curve",
    size = "Games"
  ) +
  theme_minimal(base_size = 12) +
  theme(
    panel.grid.minor = ggplot2::element_blank(),
    plot.title = ggplot2::element_text(face = "bold"),
    legend.position = "bottom"
  )

## ----vignette-options-restore, include = FALSE--------------------------------
options(.old_opts)

#' Simulate real-time probabilistic forecasts (paper designs)
#'
#' Generates replicated “games” and two competing forecast trajectories on a
#' uniform grid from 0 to 1, following the simulation constructions used in
#' Yeh, Rice, and Dubin (2022) (see `sanity_generator` / simulation scripts in
#' the replication repository). Ornstein–Uhlenbeck (`type = "OU"`) or Brownian
#' motion (`type = "BM"`) noise can drive the latent processes; outputs include
#' binary outcomes `Y` and forecast probabilities `phat_A`, `phat_B` suitable
#' for [`calc_Z()`], [`calc_eig()`], etc.
#'
#' Requires the **sde** package (Suggests) for Brownian paths.
#'
#' @param N Number of interior time steps (grid has `N+1` points from 0 to 1).
#' @param Ngame Number of independent replicates (games).
#' @param type `"OU"` (default) or `"BM"` for the innovation process used in the
#'   bivariate construction.
#' @param a,b Constants controlling the drift of the latent trajectory (see
#'   replication code).
#'
#' @return A tibble with columns including `game`, `grid`, `Y`, `phat_A`,
#'   `phat_B`, and latent `W1`, `W2`, etc., in long format.
#'
#' @references
#' Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). *The American Statistician*,
#' *76*, 214--223. \doi{10.1080/00031305.2021.1967781}
#'
#' Preprint: \url{https://arxiv.org/abs/2010.00781}
#'
#' @importFrom MASS mvrnorm
#'
#' @examples
#' \donttest{
#' if (requireNamespace("sde", quietly = TRUE)) {
#'   d <- df_gen(N = 15L, Ngame = 12L)
#'   head(d)
#' }
#' }
#'
#' @export
df_gen <- function(N, Ngame, type = c("OU", "BM"), a = 1.0, b = 0.27) {
  if (!is.numeric(N) || length(N) != 1L || N < 1L) {
    stop("`N` must be a positive integer (number of interior time steps).", call. = FALSE)
  }
  if (!is.numeric(Ngame) || length(Ngame) != 1L || Ngame < 1L) {
    stop("`Ngame` must be a positive integer (number of replicates).", call. = FALSE)
  }
  if (!requireNamespace("sde", quietly = TRUE)) {
    stop("Package \"sde\" is required for df_gen(). Install with install.packages(\"sde\").", call. = FALSE)
  }
  type <- match.arg(type)
  N <- as.integer(N)
  Ngame <- as.integer(Ngame)

  uu <- a * runif(Ngame, -1, +1) + b
  tt <- seq(0, 1, 1 / N)
  xt <- sapply(seq_len(Ngame), function(g) {
    wt <- sde::BM(x = 0, t0 = 0, T = 1, N = N) %>% as.numeric()
    uu[g] * tt + wt
  })

  Yn <- ifelse(xt[N + 1, ] > 0, 1, 0)

  if (type == "OU") {
    sim_OU <- function(n_time, n_rep) {
      sig <- 1
      tj <- seq(0, 1, 1 / n_time)
      mu <- rep(0, length(tj))
      etj <- exp(tj)
      CovXt <- sig^2 * exp(-0.5 * outer(tj, tj, `+`)) * outer(etj, etj, pmin)
      mvrnorm(n = n_rep, mu = mu, Sigma = CovXt) %>% t()
    }

    w1 <- sim_OU(n_time = N, n_rep = Ngame)
    w2 <- sim_OU(n_time = N, n_rep = Ngame)
  } else {
    w1 <- sapply(seq_len(Ngame), function(g) {
      sde::BM(x = 0, t0 = 0, T = 1, N = N) %>% as.numeric()
    })
    w2 <- sapply(seq_len(Ngame), function(g) {
      sde::BM(x = 0, t0 = 0, T = 1, N = N) %>% as.numeric()
    })
  }
  lapply(1:(N + 1), function(k) {
    tibble(game = 1:Ngame, u = uu, Xt = xt[k, ],
           grid = (k - 1) / N, Y = Yn,
           W1 = w1[k, ], W2 = w2[k, ]) # each sample point
  }) %>% list.rbind() %>% mutate(signXt = sign(Xt),
                                 phat_A = pnorm(u * (1 - grid) + Xt + W1,
                                                sd = sqrt(1 - grid)),
                                 phat_B = pnorm(u * (1 - grid) + Xt + W2,
                                                sd = sqrt(1 - grid)))
}

#' Monte Carlo p-value and quantiles for the delta test
#'
#' Given leading eigenvalues of the covariance operator used in the delta test
#' (from [`calc_eig()`]) and the observed statistic \(Z\) (from [`calc_Z()`]),
#' draws a Monte Carlo sample from the weighted sum of chi-square(1) variables
#' and returns the right-tail *p*-value and quantiles of the null distribution
#' (as in the paper’s replication code).
#'
#' @param Z Observed test statistic from [`calc_Z()`].
#' @param eig Numeric vector of leading eigenvalues (length `n_eig`).
#' @param quan Probabilities for which to return quantiles of the simulated null
#'   statistic (e.g. `c(0.90, 0.95, 0.99)`).
#' @param n_MC Number of Monte Carlo draws (default `5000`).
#'
#' @details
#' Monte Carlo draws use R's current random-number stream to calculate the p-value
#'
#' @return A list with `p_val` (one-sided p-value) and `quantile` (named
#'   vector of quantiles at `quan`).
#'
#' @references
#' Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). *The American Statistician*,
#' *76*, 214--223. \doi{10.1080/00031305.2021.1967781}
#'
#' Preprint: \url{https://arxiv.org/abs/2010.00781}
#'
#' @examples
#' \donttest{
#' if (requireNamespace("sde", quietly = TRUE)) {
#'   library(dplyr)
#'   nsamp <- 20L
#'   ngame <- 25L
#'   df <- df_gen(N = nsamp, Ngame = ngame) %>%
#'     dplyr::group_by(grid) %>%
#'     dplyr::mutate(
#'       p_bar_12 = mean(phat_A - phat_B),
#'       diff_non_cent = phat_A - phat_B,
#'       diff_cent = phat_A - phat_B - p_bar_12
#'     ) %>%
#'     dplyr::ungroup()
#'   Z <- calc_Z(df, "phat_A", "phat_B", "Y", "grid", nsamp, ngame)
#'   eig <- calc_eig(df, n_eig = 5L, ngame = ngame, nsamp = nsamp, cent = FALSE)
#'   calc_pval(Z, eig, quan = c(0.9, 0.95), n_MC = 300L)
#' }
#' }
#'
#' @export
calc_pval <- function(Z, eig, quan, n_MC = 5000) {
  n_eig <- length(eig)
  MC <- sapply(1:n_MC, function(x) {
    crossprod(eig, stats::rchisq(n_eig, df = 1))
  })

  list(
    p_val = 1 - stats::ecdf(MC)(Z),
    quantile = stats::quantile(MC, quan)
  )
}

#' Leading eigenvalues for the delta test covariance
#'
#' Forms the empirical covariance matrix of either centered or non-centered
#' forecast differences across time points (see `diff_cent` vs `diff_non_cent`
#' in the paper), and returns the leading eigenvalues scaled by one over
#' `nsamp`, for use with [`calc_pval()`]. This matches the construction in the
#' replication `utility.R` (`calc_p_val` / `eigs_sym`).
#'
#' @param df Data frame containing `grid` and a column `diff_cent` or
#'   `diff_non_cent` (vector differences of forecasts across games, aligned
#'   within each time point).
#' @param n_eig Number of leading eigenvalues to extract (dimension D in the
#'   paper).
#' @param ngame Number of independent games (used for scaling inner products).
#' @param nsamp Number of time grid points.
#' @param grid Name of the time grid column.
#' @param cent If `TRUE`, use `diff_cent` (centered differences); if `FALSE`,
#'   use `diff_non_cent`.
#'
#' @return A numeric vector of length `n_eig` of eigenvalues (descending).
#'
#' @references
#' Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). *The American Statistician*,
#' *76*, 214--223. \doi{10.1080/00031305.2021.1967781}
#'
#' Preprint: \url{https://arxiv.org/abs/2010.00781}
#'
#' @examples
#' library(dplyr)
#' library(rtForecastEval)
#' ngame <- 8L
#' nsamp <- 6L
#' gr <- seq(0, 1, length.out = nsamp)
#' df <- tidyr::expand_grid(grid = gr, game = seq_len(ngame)) %>%
#'   mutate(
#'     phat_A = stats::runif(dplyr::n()),
#'     phat_B = stats::runif(dplyr::n()),
#'     diff_non_cent = phat_A - phat_B
#'   )
#' calc_eig(df, n_eig = 4L, ngame = ngame, nsamp = nsamp, cent = FALSE)
#'
#' @export
calc_eig <- function(df, n_eig = 10, ngame, nsamp, grid = "grid", cent = FALSE) {
  diff <- ifelse(cent, "diff_cent", "diff_non_cent")
  df_list <- df %>% dplyr::select(!!sym(grid), !!sym(diff)) %>%
    group_split(!!sym(grid), .keep = FALSE)
  df_vec <- lapply(seq_along(df_list), function(x) {
    df_list[[x]] %>% unlist() %>% as.vector()
  })
  rm(df_list)
  cov_mat <- rlist::list.rbind(lapply(1:nsamp, function(i) {
    sapply(1:nsamp, function(j) {
      as.numeric(df_vec[[i]] %*% df_vec[[j]] / ngame)
    })
  }))
  eigV <- RSpectra::eigs_sym(
    A = cov_mat, k = n_eig, which = "LM",
    opts = list(retvec = FALSE)
  )$values / nsamp

  eigV
}

#' Delta test statistic for comparing two forecasting methods
#'
#' Computes the global test statistic Z for comparing two real-time forecasters
#' under squared loss: Z equals (n_game / n_samp) times the sum over grid
#' points of the squared pointwise mean loss differences, matching the
#' implementation in the paper replication code (utility.R).
#'
#' @param df Data frame containing forecasts, outcomes, and `grid`.
#' @param pA,pB Names of columns with the two probability forecasts.
#' @param Y Name of the binary outcome column.
#' @param grid Name of the column with normalized times between 0 and 1.
#' @param nsamp Number of distinct time points (length of the grid).
#' @param ngame Number of independent replicates (e.g. games).
#' @param L Loss function; default is squared error.
#'
#' @return A single numeric value of the test statistic Z.
#'
#' @references
#' Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). *The American Statistician*,
#' *76*, 214--223. \doi{10.1080/00031305.2021.1967781}
#'
#' Preprint: \url{https://arxiv.org/abs/2010.00781}
#'
#' @examples
#' library(dplyr)
#' library(rtForecastEval)
#' ngame <- 8L
#' nsamp <- 6L
#' gr <- seq(0, 1, length.out = nsamp)
#' df <- tidyr::expand_grid(grid = gr, game = seq_len(ngame)) %>%
#'   mutate(
#'     phat_A = stats::runif(dplyr::n()),
#'     phat_B = stats::runif(dplyr::n()),
#'     Y = stats::rbinom(dplyr::n(), 1L, 0.5)
#'   )
#' calc_Z(df, "phat_A", "phat_B", "Y", "grid", nsamp = nsamp, ngame = ngame)
#'
#' @export
calc_Z <- function(df, pA = "phat_1", pB = "phat_2", Y = "Y", grid = "grid",
                   nsamp, ngame,
                   L = function(x, y) (x - y)^2) {
  df %>% dplyr::select(!!sym(grid), !!sym(Y), !!sym(pA), !!sym(pB)) %>%
    group_by(!!sym(grid)) %>%
    summarise(delta_n = mean(L(!!sym(pA), !!sym(Y)) - L(!!sym(pB), !!sym(Y))), .groups = "drop") %>%
    summarise(sum(delta_n^2) / nsamp * ngame) %>% pull()
}

#' Linear interpolation of a forecast trajectory onto a grid
#'
#' Maps a vector of probabilities `prob` observed at times `grid` onto an
#' equally spaced grid of length `length(grid)` between 0 and 1 using linear
#' interpolation (`stats::approx`). Useful when aligning ESPN or model outputs
#' to the common grid used in the NBA analysis (see replication `pre_process` /
#' `utility.R`).
#'
#' @param prob Numeric vector of forecast probabilities.
#' @param grid Numeric vector of time points (same length as `prob`) between 0
#'   and 1.
#' @param outcome Scalar binary outcome \(Y\) to attach to every interpolated row.
#'
#' @return A tibble with columns `phat_approx`, `grid`, and `Y`.
#'
#' @references
#' Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). *The American Statistician*,
#' *76*, 214--223. \doi{10.1080/00031305.2021.1967781}
#'
#' Preprint: \url{https://arxiv.org/abs/2010.00781}
#'
#' @examples
#' library(rtForecastEval)
#' g <- seq(0, 1, length.out = 9L)
#' lin_interp(prob = seq(0.1, 0.9, length.out = 9L), grid = g, outcome = 1L)
#'
#' @export
lin_interp <- function(prob, grid, outcome) {
  ngrid <- length(grid)
  my_grid <- seq(0, 1, 1 / ngrid)
  df <- tibble(prob = prob, grid = grid)

  df %>%
    summarise(
      phat_approx = list(approx(grid, prob, my_grid, method = "linear")$y),
      grid = list(my_grid),
      Y = outcome
    ) %>%
    tidyr::unnest(cols = c("phat_approx", "grid"))
}

#' Pointwise mean loss difference and variance
#'
#' For each time grid point, estimates the pointwise difference in expected
#' squared (Brier) loss between two probabilistic forecasts, and the variance
#' factor used for inference (Yeh, Rice, and Dubin, 2022). With default loss
#' `L(x,y) = (x-y)^2`, the function computes the mean over games of
#' `L(Y, phat_1) - L(Y, phat_2)` at each time, and the influence-style terms
#' `si` used to form sigma-squared over four, matching the paper replication
#' code.
#'
#' @param df A data frame with one row per (game × time) in long format.
#' @param pA,pB Column names for the two forecast vectors (probabilities between
#'   0 and 1).
#' @param Y Column name for the binary outcome (0/1).
#' @param grid Column name for the normalized time grid between 0 and 1.
#' @param L Loss function; default is squared error, \(L(x,y)=(x-y)^2\).
#'
#' @return A tibble with one row per `grid` value and columns `L` (mean loss
#'   difference), `sigma2` (variance factor), and `n` (number of rows).
#'
#' @references
#' Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). *The American Statistician*,
#' *76*, 214--223. \doi{10.1080/00031305.2021.1967781}
#'
#' Preprint: \url{https://arxiv.org/abs/2010.00781}
#'
#' @examples
#' library(dplyr)
#' library(rtForecastEval)
#' ngame <- 8L
#' nsamp <- 6L
#' gr <- seq(0, 1, length.out = nsamp)
#' df <- tidyr::expand_grid(grid = gr, game = seq_len(ngame)) %>%
#'   mutate(
#'     phat_A = stats::runif(dplyr::n()),
#'     phat_B = stats::runif(dplyr::n()),
#'     Y = stats::rbinom(dplyr::n(), 1L, 0.5)
#'   )
#' calc_L_s2(df, "phat_A", "phat_B", "Y", "grid")
#'
#' @export
calc_L_s2 <- function(df, pA = "phat_1", pB = "phat_2", Y = "Y",
                      grid = "grid", L = function(x, y) (x - y)^2) {
  df %>% mutate(si = L(1, !!sym(pA)) - L(0, !!sym(pA)) -
                   (L(1, !!sym(pB)) - L(0, !!sym(pB)))) %>%
    group_by(!!sym(grid)) %>%
    summarise(
      L = mean(L(!!sym(Y), !!sym(pA)) - L(!!sym(Y), !!sym(pB))),
      sigma2 = mean(si ^ 2) / 4,
      n = n(),
      .groups = "drop"
    )
}

#' Naive pointwise confidence band for mean loss difference
#'
#' Plots the pointwise mean loss difference from [`calc_L_s2()`] (column `L`)
#' against time, with a normal-theory band using `sigma2` and the point sample
#' size `n`. This is the **naive** \(t\)-style band from the paper; global
#' inference uses [`calc_Z()`] / [`calc_pval()`] instead.
#'
#' This plot summarizes **relative skill** (loss difference) over time — not a
#' classical **calibration** plot (forecast vs observed event rate). For a
#' simple reliability-style view from the same long-format data, see the package
#' vignette example that bins `phat` and plots mean `Y` vs mean forecast.
#'
#' @param df Output of [`calc_L_s2()`] (or compatible tibble with `grid`, `L`,
#'   `sigma2`, `n`).
#' @param grid Name of the x-axis column (default `"grid"`).
#' @param L Name of the pointwise mean difference column (default `"L"`).
#' @param var Name of the variance column (default `"sigma2"`).
#' @param title,subtitle,caption Passed to [ggplot2::labs()]. Defaults describe
#'   the plot when `NULL` (some labels are omitted if `NULL`).
#' @param xlab,ylab Axis labels (see [ggplot2::labs()]).
#'
#' @return A **ggplot2** object.
#'
#' @references
#' Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). *The American Statistician*,
#' *76*, 214--223. \doi{10.1080/00031305.2021.1967781}
#'
#' Preprint: \url{https://arxiv.org/abs/2010.00781}
#'
#' @examples
#' library(dplyr)
#' library(rtForecastEval)
#' ngame <- 8L
#' nsamp <- 6L
#' gr <- seq(0, 1, length.out = nsamp)
#' df <- tidyr::expand_grid(grid = gr, game = seq_len(ngame)) %>%
#'   mutate(
#'     phat_A = stats::runif(dplyr::n()),
#'     phat_B = stats::runif(dplyr::n()),
#'     Y = stats::rbinom(dplyr::n(), 1L, 0.5)
#'   )
#' tmp <- calc_L_s2(df, "phat_A", "phat_B", "Y", "grid")
#' plot_pcb(tmp)
#'
#' @export
plot_pcb <- function(df, grid = "grid", L = "L", var = "sigma2",
                     title = "Pointwise mean loss difference (A vs B)",
                     subtitle = "Naive normal band (95%); use calc_pval() for global test",
                     caption = NULL,
                     xlab = "Normalized time (grid)",
                     ylab = "Mean squared loss difference") {
  df %>%
    ggplot(aes(!!sym(grid), !!sym(L))) +
    geom_ribbon(
      aes(
        ymax = !!sym(L) + stats::qnorm(0.975) * sqrt(!!sym(var)) / sqrt(!!sym("n")),
        ymin = !!sym(L) + stats::qnorm(0.025) * sqrt(!!sym(var)) / sqrt(!!sym("n"))
      ),
      alpha = 0.22,
      fill = "steelblue"
    ) +
    geom_line(color = "steelblue", linewidth = 0.9) +
    geom_hline(yintercept = 0, colour = "grey35", linewidth = 0.6, linetype = "dashed") +
    labs(title = title, subtitle = subtitle, caption = caption, x = xlab, y = ylab) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      panel.grid.minor = ggplot2::element_blank(),
      plot.title = ggplot2::element_text(face = "bold")
    )
}

---
title: "rtForecastEval guide"
subtitle: "EVALuating Real-Time Probabilistic Forecast"
author: |
  | Chi-Kuang Yeh (Georgia State University)
  | Gregory Rice, Joel A. Dubin (University of Waterloo)
date: "`r Sys.Date()`"
output:
  rmarkdown::html_document:
    toc: yes
    toc_float: true
    theme: readable
    highlight: tango
  rmarkdown::html_vignette:
    toc: yes
vignette: >
  %\VignetteIndexEntry{rtForecastEval guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
.old_opts <- options(digits = 3)
```

## Mind map and workflow

**rtForecastEval** compares two real-time forecasters on a common time grid. The schematic below matches the analysis order in this vignette: prepare a long tibble, run the **global delta test** (`calc_Z` → `calc_eig` → `calc_pval`), and optionally summarize **pointwise** loss with `calc_L_s2` and `plot_pcb`.

```{r workflow-schematic, echo=FALSE, warning = FALSE, message= FALSE, fig.width=8, fig.height=4, fig.cap="Typical workflow: global test (top) and pointwise loss plot (bottom)."}
library(ggplot2)

box <- function(x, y, w, h, label) {
  data.frame(
    xmin = x - w / 2, xmax = x + w / 2,
    ymin = y - h / 2, ymax = y + h / 2,
    label = label, x = x, y = y
  )
}

top <- rbind(
  box(1.2, 5, 1.1, 0.55, "Data\n(df_gen or yours)"),
  box(3.1, 5, 1.0, 0.55, "Long format +\ncentered diffs"),
  box(4.9, 5.45, 0.85, 0.45, "calc_Z"),
  box(4.9, 4.55, 0.85, 0.45, "calc_eig"),
  box(6.8, 5, 0.95, 0.55, "calc_pval")
)
bot <- rbind(
  box(3.1, 2.6, 1.0, 0.55, "calc_L_s2"),
  box(5.0, 2.6, 0.85, 0.55, "plot_pcb")
)
rects <- rbind(top, bot)

seg <- data.frame(
  x = c(1.75, 2.65, 4.45, 4.45, 5.35, 5.35, 3.1, 3.65),
  y = c(5, 5, 5.45, 4.55, 5.45, 4.55, 4.45, 2.6),
  xend = c(2.6, 3.6, 5.35, 5.35, 6.35, 6.35, 3.1, 4.55),
  yend = c(5, 5, 5.45, 4.55, 5, 5, 3.15, 2.6)
)

ggplot() +
  geom_rect(
    data = rects,
    aes(xmin = xmin, xmax = xmax, ymin = ymin, ymax = ymax),
    fill = "grey96", color = "grey35", size = 0.35
  ) +
  geom_text(data = rects, aes(x = x, y = y, label = label), size = 3) +
  geom_segment(
    data = seg,
    aes(x = x, y = y, xend = xend, yend = yend),
    arrow = grid::arrow(length = grid::unit(0.12, "cm"), type = "closed"),
    size = 0.35,
    color = "grey25"
  ) +
  annotate("text", x = 4.9, y = 5.95, label = "Global delta test (chi-square approx.)", size = 3.2, fontface = "italic") +
  annotate("text", x = 4.0, y = 1.85, label = "Pointwise mean loss + naive band", size = 3.2, fontface = "italic") +
  coord_cartesian(xlim = c(0.3, 7.5), ylim = c(1.5, 6.2), clip = "off") +
  theme_void() +
  theme(plot.margin = grid::unit(c(12, 8, 8, 8), "pt"))
```


## Figures in the paper and where they are produced

The [paper](https://doi.org/10.1080/00031305.2021.1967781) (preprint [arXiv:2010.00781](https://arxiv.org/abs/2010.00781)) has two kinds of graphics:

## NBA application: calibration surfaces, reliability diagrams, and model comparisons

Those figures are **not** self-contained in **rtForecastEval** because they require the scraped NBA play-by-play inputs, the `load_nba_data()` / `pre_process()` pipeline, and several bespoke plotting scripts.

Reproduce them from the separate replication repository (**RTPForNBA**; historically bundled with the paper’s supplementary code), not from this package alone:

| Idea in the paper | Typical script (replication repo) | Depends on |
|-------------------|-----------------------------------|------------|
| Calibration **surface** (3D: time × forecast × centered residual) | `plotting/surfacePlot.R` | `nba_FD.R` (fits logit, builds `my_df`), **plot3D**, **akima**, **lattice**, binned CIs |
| Reliability-style **plot** at one time (e.g. mid-game) | `plotting/calibrationPlot.R` (see also end of `surfacePlot.R`) | Same prepared `df_try` object |
| Skill curves / $\hat\Delta_n(t)$ for ESPN vs models (PgRS, LTW, etc.) | `plotting/1_PgRS.R`, `2_LTW.R`, `3_CF.R`, … | `utility.R` (`calc_L_s2`, `L_smoothing`), `nba_FD.R` |
| **Simulation** figures (oracle, score difference, etc.) | `simulation/PF-sim/plot_sim_*.R` | `simulation/PF-sim/simulator.R` and generated outputs |

**Suggested order in the replication repo:** prepare data under `data/`, run `nba_FD.R` (or the season scripts it sources), then `source()` the relevant file under `plotting/` with working directory set to that project root.

The rest of this vignette focuses on (1).

```{r load, message = FALSE, include = FALSE}
require(dplyr)
require(tidyr)
require(gridExtra)
require(RSpectra)
require(rlist)
```

```{r, include = FALSE}
library(rtForecastEval)
```

```{r}
library(ggplot2)
library(tibble)
library(MASS)
nsamp <- 100 # number of in-game events
ngame <- 200 # number of games (README example uses the same for comparable figures)

#' Parameter for generating the eigenvalues, and p-values
D <- 10 # Number of eigenvalues to keep
N_MC <- 5000 # for simulating the p-value
L <- function(x, y) {
  return((x - y) ^ 2)
}

# Data generation ---------------------------------------------------------
df_equ <- df_gen(N = nsamp, Ngame = ngame) %>%
  group_by(grid) %>%
  mutate(
    p_bar_12 = mean(phat_A - phat_B),
    diff_non_cent = phat_A - phat_B,
    diff_cent = phat_A - phat_B - p_bar_12
  ) %>% ungroup()

# Apply our test (explicit construction, as in utility.R) ----------------

Z <- df_equ %>% group_by(grid) %>%
  summarise(delta_n = mean(L(phat_A, Y) - L(phat_B, Y))) %>%
  {
    sum((.)$delta_n^2) / nsamp * ngame
  }

temp <- df_equ %>% group_split(grid, .keep = FALSE)

eigV_hat <- lapply(1:nsamp, function(i) {
  sapply(1:nsamp, function(j) {
    as.numeric(temp[[i]]$diff_non_cent %*% temp[[j]]$diff_non_cent / ngame)
  })
}) %>% list.rbind() %>% {
  RSpectra::eigs_sym(
    A = (.),
    k = D,
    which = "LM",
    opts = list(retvec = FALSE)
  )$values
} %>%
  {
    (.)/nsamp
  }

eigV_til <- lapply(1:nsamp, function(i) {
  sapply(1:nsamp, function(j) {
    as.numeric(temp[[i]]$diff_cent %*% temp[[j]]$diff_cent / ngame)
  })
}) %>% list.rbind() %>% {
  RSpectra::eigs_sym(
    A = (.),
    k = D,
    which = "LM",
    opts = list(retvec = FALSE)
  )$values
} %>%
  {
    (.)/nsamp
  }

MC_hat <- sapply(1:N_MC, function(x) {
  crossprod(eigV_hat, rchisq(D, df = 1))
})

q_90_hat <- quantile(MC_hat, 0.90)
q_95_hat <- quantile(MC_hat, 0.95)
q_99_hat <- quantile(MC_hat, 0.99)

MC_til <- sapply(1:N_MC, function(x) {
  crossprod(eigV_til, rchisq(D, df = 1))
})

q_90_til <- quantile(MC_til, 0.90)
q_95_til <- quantile(MC_til, 0.95)
q_99_til <- quantile(MC_til, 0.99)

p_hat <- 1 - ecdf(MC_hat)(Z)

tibble(
  type = c("non-center", "center"),
  Z = rep(Z, 2),
  "pval" = c(p_hat, p_hat),
  "90%" = c(q_90_hat, q_90_til),
  "95%" = c(q_95_hat, q_95_til),
  "99%" = c(q_99_hat, q_99_til)
)
```

### Same analysis using package functions

```{r function wrappers, fig.width = 7, fig.height = 4.2, fig.cap = "Pointwise mean loss difference (A vs B) with naive normal band — simulation setting. This is a skill curve, not a calibration diagram."}
to_center <- FALSE

ZZ <- calc_Z(df = df_equ, pA = "phat_A", pB = "phat_B", Y = "Y", nsamp = nsamp, ngame = ngame)
eigg <- calc_eig(
  df = df_equ, n_eig = D, ngame = ngame,
  nsamp = nsamp, grid = "grid", cent = to_center
)
oh <- calc_pval(ZZ, eig = eigg, quan = c(0.90, 0.95, 0.99), n_MC = N_MC)

temp <- calc_L_s2(df = df_equ, pA = "phat_A", pB = "phat_B")

plot_pcb(df = temp)

tibble(
  type = ifelse(to_center, "center", "non-center"),
  Z = ZZ,
  pval = oh$p_val,
  "90%" = oh$quantile[1],
  "95%" = oh$quantile[2],
  "99%" = oh$quantile[3]
)
```

### A simple calibration (reliability) view at one time

The previous figure tracks **skill** (which forecaster loses less Brier loss over time). A complementary check is **marginal calibration**: within a narrow time slice, do predicted probabilities match observed event frequencies? **Grey vertical** segments show a **95% central range** for $\bar Y$ under **$H_0$** in each bin: $n\bar Y \sim \mathrm{Binomial}(n, p)$ with $p = \overline{\hat p}$, using **exact** `qbinom` bounds (avoiding asymmetric normal approximation + clipping that can distort small-sample bands). The paper’s NBA figures use richer calibration **surfaces** (time × forecast × residual); here we only **bin** forecasts at a single grid point (closest to mid-game, $t \approx 0.5$) and plot mean outcome vs mean forecast — a standard reliability diagram.

```{r calibration-reliability, fig.width = 7.5, fig.height = 4, fig.cap = "Binned reliability diagram at a fixed grid (closest to 0.5): mean outcome vs mean forecast for A and B; grey bars are exact 95% central Binomial range for mean(Y) under H0. Points near the diagonal indicate better marginal calibration at that time."}
g_mid <- df_equ %>%
  distinct(grid) %>%
  slice_min(order_by = abs(grid - 0.5), n = 1) %>%
  pull(grid)

slice_t <- df_equ %>%
  filter(grid == g_mid) %>%
  transmute(
    game,
    Y,
    phat_A,
    phat_B
  )

rel_long <- slice_t %>%
  tidyr::pivot_longer(c(phat_A, phat_B), names_to = "forecaster", values_to = "phat") %>%
  mutate(forecaster = ifelse(forecaster == "phat_A", "Forecaster A", "Forecaster B"))

rel_binned <- rel_long %>%
  group_by(forecaster) %>%
  mutate(bin = ntile(phat, 10)) %>%
  group_by(forecaster, bin) %>%
  summarise(
    mean_forecast = mean(phat),
    mean_outcome = mean(Y),
    n_games = dplyr::n(),
    .groups = "drop"
  ) %>%
  mutate(
    p_bin = pmin(pmax(mean_forecast, 0), 1),
    ci_lo = stats::qbinom(0.025, size = n_games, prob = p_bin) / n_games,
    ci_hi = stats::qbinom(0.975, size = n_games, prob = p_bin) / n_games
  )

ggplot(rel_binned, aes(mean_forecast, mean_outcome)) +
  geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "grey45") +
  geom_errorbar(
    aes(x = mean_forecast, ymin = ci_lo, ymax = ci_hi),
    width = 0.02,
    color = "grey55",
    alpha = 0.85,
    linewidth = 0.35
  ) +
  geom_point(aes(size = n_games), alpha = 0.9, color = "darkred") +
  facet_wrap(~forecaster, nrow = 1) +
  coord_fixed(xlim = c(0, 1), ylim = c(0, 1), ratio = 1) +
  labs(
    title = "Binned reliability (calibration) at one grid",
    subtitle = paste0(
      "Grid = ", signif(g_mid, 3),
      " (closest to 0.5); grey: exact 95% Binomial range for mean(Y) under H0"
    ),
    x = "Mean forecast in bin",
    y = "Mean outcome (Y) in bin",
    size = "Games"
  ) +
  theme_minimal(base_size = 12) +
  theme(
    panel.grid.minor = ggplot2::element_blank(),
    plot.title = ggplot2::element_text(face = "bold")
  )
```

### Reliability with shared bins (two standard diagrams)

The facet reliability figure above uses **separate** decile bins per forecaster. Here we bin the **same games** with **equal-width** cuts of the **midpoint** $m = (\hat p_A + \hat p_B)/2$, then plot **one classical reliability diagram per forecaster** (mean forecast vs mean outcome, 45° line). **Grey vertical** segments are the **exact Binomial** **95%** central range for $\bar{Y$} under **H0** at each bin’s mean forecast. Overlaying both forecasters in one square is avoided.

```{r reliability-joint-difference, fig.width = 8, fig.height = 4.2, fig.cap = "Shared midpoint bins: two standard reliability diagrams (A and B); grey bars = exact 95% Binomial range for mean(Y) under H0."}
cal_bins <- slice_t %>%
  mutate(mid = (phat_A + phat_B) / 2) %>%
  mutate(bin = cut(mid, breaks = seq(0, 1, length.out = 11), include.lowest = TRUE))

rel_joint <- cal_bins %>%
  group_by(bin) %>%
  summarise(
    mean_forecast_A = mean(phat_A),
    mean_forecast_B = mean(phat_B),
    mean_outcome = mean(Y),
    n_games = dplyr::n(),
    .groups = "drop"
  ) %>%
  filter(!is.na(bin)) %>%
  mutate(
    p_A = pmin(pmax(mean_forecast_A, 0), 1),
    p_B = pmin(pmax(mean_forecast_B, 0), 1),
    ci_lo_A = stats::qbinom(0.025, size = n_games, prob = p_A) / n_games,
    ci_hi_A = stats::qbinom(0.975, size = n_games, prob = p_A) / n_games,
    ci_lo_B = stats::qbinom(0.025, size = n_games, prob = p_B) / n_games,
    ci_hi_B = stats::qbinom(0.975, size = n_games, prob = p_B) / n_games
  )

rel_joint_long <- rel_joint %>%
  tidyr::pivot_longer(
    c(mean_forecast_A, mean_forecast_B),
    names_to = "which",
    names_prefix = "mean_forecast_",
    values_to = "mean_forecast"
  ) %>%
  mutate(
    forecaster = ifelse(which == "A", "Forecaster A", "Forecaster B"),
    ci_lo = ifelse(which == "A", ci_lo_A, ci_lo_B),
    ci_hi = ifelse(which == "A", ci_hi_A, ci_hi_B)
  )

ggplot(rel_joint_long, aes(mean_forecast, mean_outcome)) +
  geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "grey45") +
  geom_errorbar(
    aes(x = mean_forecast, ymin = ci_lo, ymax = ci_hi),
    width = 0.02,
    color = "grey55",
    alpha = 0.85,
    linewidth = 0.4
  ) +
  geom_point(aes(color = forecaster, size = n_games), alpha = 0.9) +
  facet_wrap(~forecaster, nrow = 1) +
  coord_fixed(xlim = c(0, 1), ylim = c(0, 1), ratio = 1) +
  scale_color_manual(values = c("Forecaster A" = "steelblue", "Forecaster B" = "orangered")) +
  labs(
    title = "Reliability: shared midpoint bins (one classical diagram per forecaster)",
    subtitle = "Bins from cut((phat_A + phat_B) / 2); grey = exact 95% Binomial H0 range for mean(Y)",
    x = "Mean forecast in bin",
    y = "Mean outcome (Y) in bin",
    color = NULL,
    size = "Games"
  ) +
  theme_minimal(base_size = 12) +
  theme(
    panel.grid.minor = ggplot2::element_blank(),
    plot.title = ggplot2::element_text(face = "bold"),
    legend.position = "right"
  ) +
  guides(color = "none")
```

### Comparing calibration between forecasters (error curves)

The reliability diagram shows **levels** on the 45° plot; here we plot **calibration errors** $\bar Y - \bar{\hat p}_k$ vs bin midpoint. The **dashed** line is $\bar{\hat p}_B - \bar{\hat p}_A$.

```{r calibration-compare-forecasters, fig.width = 7.5, fig.height = 4.5, fig.cap = "Mean calibration error per forecaster (solid) and difference in mean forecasts (dashed), binned by (phat_A + phat_B) / 2 at the same grid as the reliability figure."}
cal_compare <- cal_bins %>%
  group_by(bin) %>%
  summarise(
    mid_hat = mean(mid),
    cal_err_A = mean(Y) - mean(phat_A),
    cal_err_B = mean(Y) - mean(phat_B),
    n_games = dplyr::n(),
    .groups = "drop"
  ) %>%
  filter(!is.na(bin))

cal_long <- cal_compare %>%
  tidyr::pivot_longer(
    c(cal_err_A, cal_err_B),
    names_to = "which",
    values_to = "cal_err"
  ) %>%
  mutate(
    which = ifelse(which == "cal_err_A", "A: mean(Y) - phat_A", "B: mean(Y) - phat_B")
  )

ggplot(cal_long, aes(mid_hat, cal_err, color = which)) +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey70") +
  geom_line(linewidth = 0.85) +
  geom_point(aes(size = n_games), alpha = 0.85) +
  geom_line(
    data = cal_compare,
    aes(mid_hat, cal_err_B - cal_err_A),
    inherit.aes = FALSE,
    color = "grey35",
    linetype = "dashed",
    linewidth = 0.65
  ) +
  labs(
    title = "Calibration comparison (same midpoint bins)",
    subtitle = "Solid: mean calibration error per forecaster; dashed: mean(phat_B) - mean(phat_A)",
    x = "Mean (phat_A + phat_B) / 2 in bin",
    y = "mean(Y) - mean(phat)",
    color = "Curve",
    size = "Games"
  ) +
  theme_minimal(base_size = 12) +
  theme(
    panel.grid.minor = ggplot2::element_blank(),
    plot.title = ggplot2::element_text(face = "bold"),
    legend.position = "bottom"
  )
```

```{r vignette-options-restore, include = FALSE}
options(.old_opts)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_eig.R
\name{calc_eig}
\alias{calc_eig}
\title{Leading eigenvalues for the delta test covariance}
\usage{
calc_eig(df, n_eig = 10, ngame, nsamp, grid = "grid", cent = FALSE)
}
\arguments{
\item{df}{Data frame containing \code{grid} and a column \code{diff_cent} or
\code{diff_non_cent} (vector differences of forecasts across games, aligned
within each time point).}

\item{n_eig}{Number of leading eigenvalues to extract (dimension D in the
paper).}

\item{ngame}{Number of independent games (used for scaling inner products).}

\item{nsamp}{Number of time grid points.}

\item{grid}{Name of the time grid column.}

\item{cent}{If \code{TRUE}, use \code{diff_cent} (centered differences); if \code{FALSE},
use \code{diff_non_cent}.}
}
\value{
A numeric vector of length \code{n_eig} of eigenvalues (descending).
}
\description{
Forms the empirical covariance matrix of either centered or non-centered
forecast differences across time points (see \code{diff_cent} vs \code{diff_non_cent}
in the paper), and returns the leading eigenvalues scaled by one over
\code{nsamp}, for use with \code{\link[=calc_pval]{calc_pval()}}. This matches the construction in the
replication \code{utility.R} (\code{calc_p_val} / \code{eigs_sym}).
}
\examples{
library(dplyr)
library(rtForecastEval)
ngame <- 8L
nsamp <- 6L
gr <- seq(0, 1, length.out = nsamp)
df <- tidyr::expand_grid(grid = gr, game = seq_len(ngame)) \%>\%
  mutate(
    phat_A = stats::runif(dplyr::n()),
    phat_B = stats::runif(dplyr::n()),
    diff_non_cent = phat_A - phat_B
  )
calc_eig(df, n_eig = 4L, ngame = ngame, nsamp = nsamp, cent = FALSE)

}
\references{
Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). \emph{The American Statistician},
\emph{76}, 214--223. \doi{10.1080/00031305.2021.1967781}

Preprint: \url{https://arxiv.org/abs/2010.00781}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_pval.R
\name{calc_pval}
\alias{calc_pval}
\title{Monte Carlo p-value and quantiles for the delta test}
\usage{
calc_pval(Z, eig, quan, n_MC = 5000)
}
\arguments{
\item{Z}{Observed test statistic from \code{\link[=calc_Z]{calc_Z()}}.}

\item{eig}{Numeric vector of leading eigenvalues (length \code{n_eig}).}

\item{quan}{Probabilities for which to return quantiles of the simulated null
statistic (e.g. \code{c(0.90, 0.95, 0.99)}).}

\item{n_MC}{Number of Monte Carlo draws (default \code{5000}).}
}
\value{
A list with \code{p_val} (one-sided p-value) and \code{quantile} (named
vector of quantiles at \code{quan}).
}
\description{
Given leading eigenvalues of the covariance operator used in the delta test
(from \code{\link[=calc_eig]{calc_eig()}}) and the observed statistic \(Z\) (from \code{\link[=calc_Z]{calc_Z()}}),
draws a Monte Carlo sample from the weighted sum of chi-square(1) variables
and returns the right-tail \emph{p}-value and quantiles of the null distribution
(as in the paper’s replication code).
}
\details{
Monte Carlo draws use R's current random-number stream. The function does
not call \code{\link[base]{set.seed}}; call it yourself beforehand if you
need reproducible \code{p_val} and quantiles.
}
\examples{
\donttest{
if (requireNamespace("sde", quietly = TRUE)) {
  library(dplyr)
  nsamp <- 20L
  ngame <- 25L
  df <- df_gen(N = nsamp, Ngame = ngame) \%>\%
    dplyr::group_by(grid) \%>\%
    dplyr::mutate(
      p_bar_12 = mean(phat_A - phat_B),
      diff_non_cent = phat_A - phat_B,
      diff_cent = phat_A - phat_B - p_bar_12
    ) \%>\%
    dplyr::ungroup()
  Z <- calc_Z(df, "phat_A", "phat_B", "Y", "grid", nsamp, ngame)
  eig <- calc_eig(df, n_eig = 5L, ngame = ngame, nsamp = nsamp, cent = FALSE)
  set.seed(1)
  calc_pval(Z, eig, quan = c(0.9, 0.95), n_MC = 300L)
}
}

}
\references{
Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). \emph{The American Statistician},
\emph{76}, 214--223. \doi{10.1080/00031305.2021.1967781}

Preprint: \url{https://arxiv.org/abs/2010.00781}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_L_s2.R
\name{calc_L_s2}
\alias{calc_L_s2}
\title{Pointwise mean loss difference and variance}
\usage{
calc_L_s2(
  df,
  pA = "phat_1",
  pB = "phat_2",
  Y = "Y",
  grid = "grid",
  L = function(x, y) (x - y)^2
)
}
\arguments{
\item{df}{A data frame with one row per (game × time) in long format.}

\item{pA, pB}{Column names for the two forecast vectors (probabilities between
0 and 1).}

\item{Y}{Column name for the binary outcome (0/1).}

\item{grid}{Column name for the normalized time grid between 0 and 1.}

\item{L}{Loss function; default is squared error, \(L(x,y)=(x-y)^2\).}
}
\value{
A tibble with one row per \code{grid} value and columns \code{L} (mean loss
difference), \code{sigma2} (variance factor), and \code{n} (number of rows).
}
\description{
For each time grid point, estimates the pointwise difference in expected
squared (Brier) loss between two probabilistic forecasts, and the variance
factor used for inference (Yeh, Rice, and Dubin, 2022). With default loss
\code{L(x,y) = (x-y)^2}, the function computes the mean over games of
\code{L(Y, phat_1) - L(Y, phat_2)} at each time, and the influence-style terms
\code{si} used to form sigma-squared over four, matching the paper replication
code.
}
\examples{
library(dplyr)
library(rtForecastEval)
ngame <- 8L
nsamp <- 6L
gr <- seq(0, 1, length.out = nsamp)
df <- tidyr::expand_grid(grid = gr, game = seq_len(ngame)) \%>\%
  mutate(
    phat_A = stats::runif(dplyr::n()),
    phat_B = stats::runif(dplyr::n()),
    Y = stats::rbinom(dplyr::n(), 1L, 0.5)
  )
calc_L_s2(df, "phat_A", "phat_B", "Y", "grid")

}
\references{
Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). \emph{The American Statistician},
\emph{76}, 214--223. \doi{10.1080/00031305.2021.1967781}

Preprint: \url{https://arxiv.org/abs/2010.00781}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_pcb.R
\name{plot_pcb}
\alias{plot_pcb}
\title{Naive pointwise confidence band for mean loss difference}
\usage{
plot_pcb(
  df,
  grid = "grid",
  L = "L",
  var = "sigma2",
  title = "Pointwise mean loss difference (A vs B)",
  subtitle = "Naive normal band (95\%); use calc_pval() for global test",
  caption = NULL,
  xlab = "Normalized time (grid)",
  ylab = "Mean squared loss difference"
)
}
\arguments{
\item{df}{Output of \code{\link[=calc_L_s2]{calc_L_s2()}} (or compatible tibble with \code{grid}, \code{L},
\code{sigma2}, \code{n}).}

\item{grid}{Name of the x-axis column (default \code{"grid"}).}

\item{L}{Name of the pointwise mean difference column (default \code{"L"}).}

\item{var}{Name of the variance column (default \code{"sigma2"}).}

\item{title, subtitle, caption}{Passed to \code{\link[ggplot2:labs]{ggplot2::labs()}}. Defaults describe
the plot when \code{NULL} (some labels are omitted if \code{NULL}).}

\item{xlab, ylab}{Axis labels (see \code{\link[ggplot2:labs]{ggplot2::labs()}}).}
}
\value{
A \strong{ggplot2} object.
}
\description{
Plots the pointwise mean loss difference from \code{\link[=calc_L_s2]{calc_L_s2()}} (column \code{L})
against time, with a normal-theory band using \code{sigma2} and the point sample
size \code{n}. This is the \strong{naive} \(t\)-style band from the paper; global
inference uses \code{\link[=calc_Z]{calc_Z()}} / \code{\link[=calc_pval]{calc_pval()}} instead.
}
\details{
This plot summarizes \strong{relative skill} (loss difference) over time — not a
classical \strong{calibration} plot (forecast vs observed event rate). For a
simple reliability-style view from the same long-format data, see the package
vignette example that bins \code{phat} and plots mean \code{Y} vs mean forecast.
}
\examples{
library(dplyr)
library(rtForecastEval)
ngame <- 8L
nsamp <- 6L
gr <- seq(0, 1, length.out = nsamp)
df <- tidyr::expand_grid(grid = gr, game = seq_len(ngame)) \%>\%
  mutate(
    phat_A = stats::runif(dplyr::n()),
    phat_B = stats::runif(dplyr::n()),
    Y = stats::rbinom(dplyr::n(), 1L, 0.5)
  )
tmp <- calc_L_s2(df, "phat_A", "phat_B", "Y", "grid")
plot_pcb(tmp)

}
\references{
Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). \emph{The American Statistician},
\emph{76}, 214--223. \doi{10.1080/00031305.2021.1967781}

Preprint: \url{https://arxiv.org/abs/2010.00781}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/df_gen.R
\name{df_gen}
\alias{df_gen}
\title{Simulate real-time probabilistic forecasts (paper designs)}
\usage{
df_gen(N, Ngame, type = c("OU", "BM"), a = 1, b = 0.27)
}
\arguments{
\item{N}{Number of interior time steps (grid has \code{N+1} points from 0 to 1).}

\item{Ngame}{Number of independent replicates (games).}

\item{type}{\code{"OU"} (default) or \code{"BM"} for the innovation process used in the
bivariate construction.}

\item{a, b}{Constants controlling the drift of the latent trajectory (see
replication code).}
}
\value{
A tibble with columns including \code{game}, \code{grid}, \code{Y}, \code{phat_A},
\code{phat_B}, and latent \code{W1}, \code{W2}, etc., in long format.
}
\description{
Generates replicated “games” and two competing forecast trajectories on a
uniform grid from 0 to 1, following the simulation constructions used in
Yeh, Rice, and Dubin (2022) (see \code{sanity_generator} / simulation scripts in
the replication repository). Ornstein–Uhlenbeck (\code{type = "OU"}) or Brownian
motion (\code{type = "BM"}) noise can drive the latent processes; outputs include
binary outcomes \code{Y} and forecast probabilities \code{phat_A}, \code{phat_B} suitable
for \code{\link[=calc_Z]{calc_Z()}}, \code{\link[=calc_eig]{calc_eig()}}, etc.
}
\details{
Requires the \strong{sde} package (Suggests) for Brownian paths.
}
\examples{
\donttest{
if (requireNamespace("sde", quietly = TRUE)) {
  d <- df_gen(N = 15L, Ngame = 12L)
  head(d)
}
}

}
\references{
Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). \emph{The American Statistician},
\emph{76}, 214--223. \doi{10.1080/00031305.2021.1967781}

Preprint: \url{https://arxiv.org/abs/2010.00781}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rtForecastEval-package.R
\docType{package}
\name{rtForecastEval-package}
\alias{rtForecastEval-package}
\alias{rtForecastEval}
\alias{Implements}
\alias{the}
\alias{**delta}
\alias{test**}
\alias{and}
\alias{related}
\alias{tools}
\alias{from}
\alias{Yeh,}
\alias{Rice,}
\alias{Dubin}
\alias{for}
\alias{comparing}
\alias{two}
\alias{continuously}
\alias{updated}
\alias{probabilistic}
\alias{forecasts}
\alias{(e.g.}
\alias{in-game}
\alias{win}
\alias{probabilities)}
\alias{under}
\alias{squared}
\alias{(Brier-type)}
\alias{loss.}
\alias{Core}
\alias{steps:}
\alias{estimate}
\alias{pointwise}
\alias{mean}
\alias{loss}
\alias{difference}
\alias{its}
\alias{variance}
\alias{([`calc_L_s2()`]),}
\alias{form}
\alias{test}
\alias{statistic}
\alias{[`calc_Z()`],}
\alias{leading}
\alias{eigenvalues}
\alias{of}
\alias{associated}
\alias{covariance}
\alias{[`calc_eig()`],}
\alias{obtain}
\alias{Monte}
\alias{Carlo}
\alias{p-values}
\alias{quantiles}
\alias{[`calc_pval()`].}
\alias{[`df_gen()`]}
\alias{reproduces}
\alias{simulation}
\alias{designs}
\alias{used}
\alias{in}
\alias{paper;}
\alias{[`plot_pcb()`]}
\alias{draws}
\alias{naive}
\alias{confidence}
\alias{bands.}
\alias{The}
\alias{companion}
\alias{replication}
\alias{repository}
\alias{(NBA}
\alias{data}
\alias{workflow,}
\alias{calibration}
\alias{surfaces,}
\alias{additional}
\alias{plots)}
\alias{is}
\alias{separate}
\alias{this}
\alias{package;}
\alias{package}
\alias{exports}
\alias{statistical}
\alias{building}
\alias{blocks}
\alias{there.}
\title{rtForecastEval: Compare real-time probabilistic forecasts}
\description{
Methods from Yeh, Rice, and Dubin (2022) \doi{10.1080/00031305.2021.1967781} for comparing two continuously updated probabilistic forecasts under squared (Brier) loss: pointwise loss and variance, a global delta test (Monte Carlo p-values), simulation designs, and a naive pointwise band plot.
}
\references{
Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). Evaluating real-time
probabilistic forecasts with application to National Basketball Association
outcome prediction. \emph{The American Statistician}, \emph{76}, 214--223.
\doi{10.1080/00031305.2021.1967781}

Preprint: \url{https://arxiv.org/abs/2010.00781} (PDF:
\url{https://arxiv.org/pdf/2010.00781.pdf}).
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/chikuang/rtForecastEval}
  \item Report bugs at \url{https://github.com/chikuang/rtForecastEval/issues}
}

}
\author{
\strong{Maintainer}: Chi-Kuang Yeh \email{cyeh@gsu.edu} (\href{https://orcid.org/0000-0001-7057-2096}{ORCID})

Other contributors:
\itemize{
  \item Gregory Rice [contributor, thesis advisor]
  \item Joel A. Dubin [contributor, thesis advisor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lin_interp.R
\name{lin_interp}
\alias{lin_interp}
\title{Linear interpolation of a forecast trajectory onto a grid}
\usage{
lin_interp(prob, grid, outcome)
}
\arguments{
\item{prob}{Numeric vector of forecast probabilities.}

\item{grid}{Numeric vector of time points (same length as \code{prob}) between 0
and 1.}

\item{outcome}{Scalar binary outcome \(Y\) to attach to every interpolated row.}
}
\value{
A tibble with columns \code{phat_approx}, \code{grid}, and \code{Y}.
}
\description{
Maps a vector of probabilities \code{prob} observed at times \code{grid} onto an
equally spaced grid of length \code{length(grid)} between 0 and 1 using linear
interpolation (\code{stats::approx}). Useful when aligning ESPN or model outputs
to the common grid used in the NBA analysis (see replication \code{pre_process} /
\code{utility.R}).
}
\examples{
library(rtForecastEval)
g <- seq(0, 1, length.out = 9L)
lin_interp(prob = seq(0.1, 0.9, length.out = 9L), grid = g, outcome = 1L)

}
\references{
Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). \emph{The American Statistician},
\emph{76}, 214--223. \doi{10.1080/00031305.2021.1967781}

Preprint: \url{https://arxiv.org/abs/2010.00781}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_Z.R
\name{calc_Z}
\alias{calc_Z}
\title{Delta test statistic for comparing two forecasting methods}
\usage{
calc_Z(
  df,
  pA = "phat_1",
  pB = "phat_2",
  Y = "Y",
  grid = "grid",
  nsamp,
  ngame,
  L = function(x, y) (x - y)^2
)
}
\arguments{
\item{df}{Data frame containing forecasts, outcomes, and \code{grid}.}

\item{pA, pB}{Names of columns with the two probability forecasts.}

\item{Y}{Name of the binary outcome column.}

\item{grid}{Name of the column with normalized times between 0 and 1.}

\item{nsamp}{Number of distinct time points (length of the grid).}

\item{ngame}{Number of independent replicates (e.g. games).}

\item{L}{Loss function; default is squared error.}
}
\value{
A single numeric value of the test statistic Z.
}
\description{
Computes the global test statistic Z for comparing two real-time forecasters
under squared loss: Z equals (n_game / n_samp) times the sum over grid
points of the squared pointwise mean loss differences, matching the
implementation in the paper replication code (utility.R).
}
\examples{
library(dplyr)
library(rtForecastEval)
ngame <- 8L
nsamp <- 6L
gr <- seq(0, 1, length.out = nsamp)
df <- tidyr::expand_grid(grid = gr, game = seq_len(ngame)) \%>\%
  mutate(
    phat_A = stats::runif(dplyr::n()),
    phat_B = stats::runif(dplyr::n()),
    Y = stats::rbinom(dplyr::n(), 1L, 0.5)
  )
calc_Z(df, "phat_A", "phat_B", "Y", "grid", nsamp = nsamp, ngame = ngame)

}
\references{
Yeh, C.-K., Rice, G., & Dubin, J. A. (2022). \emph{The American Statistician},
\emph{76}, 214--223. \doi{10.1080/00031305.2021.1967781}

Preprint: \url{https://arxiv.org/abs/2010.00781}
}
